#!/bin/bash -ux
# build errors == SKIP
VER="undefined"

clean () {
    make clean
    git clean -f -x
    rm -rf .pc/
    git checkout .
    VER=$(git describe)
}

apply () {
    pdir=${1}
    echo "Trying patches in ${pdir}"
    export QUILT_PATCHES="${pdir}/debian/patches"
    if ! quilt push -a; then
        return 1
    fi
    return 0
}

basepath="/mnt/qemubuild"
for ppath in p-2.0.0 p-2.5.0 p1 p2 p3 p4 failed; do
    clean
    if [ "${ppath}" == "failed" ]; then
        echo "We need another set of patches"
        exit 130
    fi
    if apply "${basepath}/${ppath}"; then
        break
    fi
done

skip () {
   exit 125
}
set -eE
trap skip ERR

./configure \
  --extra-cflags="-g -O2 -fstack-protector --param=ssp-buffer-size=4 -Wformat -Werror=format-security -D_FORTIFY_SOURCE=2 -DCONFIG_QEMU_DATAPATH='\"/usr/share/qemu:/usr/share/seabios:/usr/lib/ipxe/qemu\"' -DVENDOR_UBUNTU" \
  --extra-ldflags="-Wl,-Bsymbolic-functions -Wl,-z,relro -Wl,--as-needed" \
  --prefix="/usr" --sysconfdir="/etc" --libdir="/usr/lib/x86_64-linux-gnu" --libexecdir="/usr/lib/qemu" \
  --localstatedir=/var --disable-blobs --disable-strip --with-system-pixman \
  --interp-prefix="/etc/qemu-binfmt/%M" --localstatedir="/var"  --disable-user \
  --target-list=x86_64-softmmu \
  --enable-system \
  --enable-modules \
  --enable-linux-aio \
  --audio-drv-list=alsa --enable-attr \
  --enable-curl --enable-fdt \
  --enable-curses --with-system-pixman --enable-seccomp --enable-uuid --enable-xfsctl \
  --enable-kvm --enable-vhost-net \
  --disable-spice --disable-libusb --disable-usb-redir \
  --disable-vnc --disable-vnc-jpeg --disable-vnc-png \
  --disable-gtk --disable-vte --disable-sdl \
  --disable-bluez --disable-brlapi \
  --disable-linux-user \
  --disable-libiscsi \
  --disable-rbd \
  --disable-vnc-sasl \
  --disable-xen \
  --disable-vhdx \
  --disable-virtfs || exit 125
# Not available in 2.0 - just leave defaults
  # --disable-tpm \
  # --enable-gnutls \
  # --disable-smartcard \
make -j4
sudo mv x86_64-softmmu/qemu-system-x86_64 /usr/bin/qemu-system-x86_64.git-${VER}
sudo cp /usr/bin/qemu-system-x86_64.git-${VER} /usr/bin/qemu-system-x86_64

# Test the case
virsh start trusty-lvl2
sleep 30s
uvt-kvm wait --insecure trusty-lvl2
uvt-kvm ssh --insecure trusty-lvl2 'uname -r; cat /sys/devices/system/cpu/vulnerabilities/mds; cat /proc/cpuinfo | grep -e ^bug -e ^flags | grep md' || true
if uvt-kvm ssh --insecure trusty-lvl2 'cat /sys/devices/system/cpu/vulnerabilities/mds' | grep -e "^Mitigation"; then
    rc=0
else
    rc=1
fi
virsh shutdown trusty-lvl2
sleep 10s
virsh destroy trusty-lvl2 || true

# trusty's git had no terms and new/old yet
# so good/bad is inverse
#git bisect start
#git bisect good v2.0.0
#git bisect bad v2.5.0
clean
if [ ${rc} -eq 0 ]; then
    echo "Mitigation found, qemu fixed"
    exit 1 # new
else
    echo "Mitigation NOT found, qemu broken"
    exit 0 # old
fi
